import os

from dotenv import load_dotenv
from jira import JIRA
from .filewriter import *


class JiraHandler:
    def __init__(self, host=None, credentials=None, certpath="C:\\Users\\Jeferson_Dasilva\\TestGenAI\\itspipe\\cert2.crt"):
        load_dotenv()
        self.host = host
        self.credentials = credentials
        self.certpath = certpath
        self.cached_issues = None
        self.__connect()

    def fetch(self, issues_keys):
        if not issues_keys: 
            return []
        query = f"key in ({ ','.join(issues_keys) })"
        self.cached_issues = self.jiraAPI.search_issues(query, maxResults=100)
        return self.cached_issues
    
    def get_issue(self, key, refetch=False):
        if refetch or not self.cached_issues:
            self.fetch([key])
        for issue in self.cached_issues:
            if issue.key == key:
                return issue
        return None
    
    def create_issue(self, fields):
        if not fields:
            raise Exception("issue fields in JSON-format string is required")
        try:
            new_issue = self.jiraAPI.create_issue(fields)
            return new_issue
        except Exception as e:
            raise RuntimeError(e)
        
    def to_file(self, output_path, output_file_name, output_format="json"):
        if output_format == "json":
            save_to_multiple_files(self.cached_issues, output_path, output_file_name)
        else:
            save_to_file(self.cached_issues, output_format, output_file_name, output_path)

    def __connect(self):

        connection_options = {}
        if os.path.exists(self.certpath) and \
            os.path.isfile(self.certpath):
            connection_options["verify"] = self.certpath
            print("CERT exists: ", connection_options)

        if not self.host:
            self.host = self.__get_base_url()
            print(f"host: {self.host}")
        
        if not self.credentials:
            env_credentials = self.__get_env_credentials()
            print(f"env_credentials: {env_credentials}")
            self.credentials = list(env_credentials.values())
        else:
            if isinstance(self.credentials, tuple):
                self.credentials = [self.credentials]

        print(f"credentials: {self.credentials}")
        for credential_pair in self.credentials:
            try:
                self.jiraAPI = JIRA(
                    server = self.host,
                    basic_auth = credential_pair,
                    options = connection_options
                )
                print("connected to jira")
                return
            except Exception as e:
                raise ConnectionError(e)

    def __get_base_url(self):
        api_host = os.getenv("JIRA_API_HOST")
        if not api_host:
            raise EnvironmentError(f"JIRA_API_HOST env variable is missing")
        return api_host
    
    def __get_env_credentials(self):
        available_credentials = {
            "user_credentials": (
                os.getenv("JIRA_USER_NAME"), 
                os.getenv("JIRA_USER_PASSWORD")),
            "api_credentials": (
                os.getenv("JIRA_USER_EMAIL"),
                os.getenv("JIRA_USER_API_KEY")
            )
        } 

        missing_credentials_pairs = []
        for credential_type, credential_pair in available_credentials.items():
            if any(not c for c in credential_pair):
                missing_credentials_pairs.append(credential_type)

        for credential_pair in missing_credentials_pairs:
            del available_credentials[credential_pair]

        if len(available_credentials) == 0:
            raise EnvironmentError("No user credentials found")
        
        return available_credentials